"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.generateFilesFromTemplate = exports.generateCodeFromTpl = exports.writeLowCodeFiles = exports.generateAppData = exports.generateRouterFile = exports.getListenersString = exports.generateDataBind = exports.getComponentSchemaString = exports.isSlot = exports.pullComponentToListByInstance = exports.pullActionToListByInstances = exports.getOriginComponentAndActionList = exports.runGenerateCore = void 0;
const weapps_core_1 = require("../../weapps-core");
const common_1 = require("../types/common");
const path_1 = __importDefault(require("path"));
const template_1 = __importDefault(require("../template"));
const util_1 = require("../util");
const config_1 = require("../config");
const os_1 = __importDefault(require("os"));
const react_schema_renderer_1 = require("@formily/react-schema-renderer");
const style_1 = require("../util/style");
const chalk_1 = __importDefault(require("chalk"));
const types_1 = require("../../types");
const dataSource_1 = require("../../utils/dataSource");
const cals_1 = require("@cloudbase/cals");
const lodash_1 = require("lodash");
const _1 = require(".");
async function runGenerateCore(props) {
    const { appBuildDir, cals, appData, subAppDataList = [], dependencies = [], buildTypeList, appKey, basename, extraData = {
        isComposite: false,
        compProps: {},
    }, i18nConfig, fileCodeMap, isSandbox = false, deployOptions, _indexPage, processCssUnit, rpxCalcRoot, isPrivateMode = false, } = props;
    const { domain } = appData.extra;
    const allAppDataList = [appData].concat(subAppDataList);
    // 安装插件依赖
    const deps = {};
    allAppDataList.map((app) => {
        Object.entries(app.npmDependencies).forEach(([name, version]) => {
            deps[name] = version;
        });
    });
    // 合并组件库的公共npm
    dependencies.map((compLb) => {
        if (compLb.isComposite && compLb.compLibCommonResource) {
            Object.assign(deps, compLb.compLibCommonResource.npm);
        }
    });
    await Promise.all(allAppDataList.map(async (data) => {
        const { pageInstanceList, rootPath = '' } = data;
        console.log(chalk_1.default.blue.bold(`Generating files for ${rootPath ? `Sub app ${rootPath}` : 'Main app'}`));
        const dstDir = path_1.default.join(appBuildDir, 'src', rootPath ? `${rootPath}` : '');
        /**
         * pages 文件无需单独生成
         * pages 文件由 render 基类动态生成
         */
        // await Promise.all(
        //   pageInstanceList.map((pageInstance) =>
        //     generateSinglePageJsxFile(
        //       pageInstance,
        //       appBuildDir,
        //       rootPath,
        //       dependencies,
        //       extraData,
        //       buildTypeList,
        //       fileCodeMap,
        //       isSandbox,
        //       processCssUnit,
        //       rpxCalcRoot,
        //     ),
        //   ),
        // );
        await generateCodeFromTpl(appKey, data, dstDir, rootPath, extraData, i18nConfig, fileCodeMap, isSandbox, subAppDataList, buildTypeList, deployOptions, domain, dependencies, processCssUnit, isPrivateMode);
        await writeLowCodeFiles(data, dstDir, fileCodeMap, rootPath, processCssUnit, rpxCalcRoot);
    }));
    await generateAppData(cals, dependencies, appBuildDir, deployOptions, fileCodeMap);
    await generateRouterFile(allAppDataList, appBuildDir, basename, buildTypeList, fileCodeMap, isSandbox, _indexPage);
}
exports.runGenerateCore = runGenerateCore;
/**
 * 根据复合组件库获取到所有包含抽象节点的数据
 * @param dependencies
 * @returns
 */
function getGenericCompFromDep(dependencies = []) {
    const genericCompMap = {};
    dependencies.forEach((compLib) => {
        if (compLib.isComposite) {
            compLib.components.forEach((component) => {
                (0, util_1.deepDealComponentSchemaJson)(Object.values(component.componentInstances)[0], (componentSchema) => {
                    if (componentSchema === null || componentSchema === void 0 ? void 0 : componentSchema.genericComp) {
                        genericCompMap[`${component.materialName}:${component.name}`] = componentSchema.genericComp;
                    }
                });
            });
        }
    });
    return genericCompMap;
}
async function generateSinglePageJsxFile(pageInstance, appBuildDir, rootPath, dependencies = [], extraData = {
    isComposite: false,
    compProps: {},
}, buildTypeList, fileCodeMap, isSandbox, processCssUnit, rpxCalcRoot) {
    const dstDir = path_1.default.join(appBuildDir, 'src', rootPath ? `packages/${rootPath}` : '');
    const { componentSchemaJson, data, style, listenerInstances: pageListenerInstances } = pageInstance;
    const genericCompMap = getGenericCompFromDep(dependencies);
    // originComponentList 包含了引入的组件与抽象节点绑定的组件
    const { originComponentList, originActionList } = getOriginComponentAndActionList(componentSchemaJson, dependencies, [], [], genericCompMap);
    pullActionToListByInstances(pageListenerInstances, originActionList, dependencies);
    const originEntryMap = {};
    originComponentList.forEach((component) => {
        if (!originEntryMap[component.materialName]) {
            const info = {
                materialName: component.materialName,
                version: component.materialVersion,
                entries: component.entries,
            };
            originEntryMap[component.materialName] = info;
        }
    });
    const useMaterialActions = [];
    originActionList.reduce((map, action) => {
        if (action.type === weapps_core_1.ActionType.Material) {
            const sourceKey = `${action.materialName}:${action.name}`;
            if (!map[sourceKey]) {
                useMaterialActions.push(action);
                map[sourceKey] = true;
            }
        }
        return map;
    }, {});
    useMaterialActions.forEach((action) => {
        if (!originEntryMap[action.materialName]) {
            const info = {
                materialName: action.materialName,
                version: action.materialVersion,
                entries: action.entries,
            };
            originEntryMap[action.materialName] = info;
        }
    });
    const entryImportStringArr = isSandbox ? [] : getEntryImportStringArr(Object.values(originEntryMap));
    const componentImportStringArr = isSandbox
        ? originComponentList.map((compItem) => {
            return `const ${(0, util_1.upperFirst)(compItem.variableName)} = window["@weapps-materials-main-${compItem.materialName}"].components["${compItem.name}"]`;
        })
        : getComponentImportStringArr(originComponentList);
    const actionImportStringArr = isSandbox
        ? useMaterialActions.map((action) => {
            return `const ${action.variableName} = window["@weapps-materials-main-${action.materialName}"].actions["${action.name}"]`;
        })
        : getActionImportStringArr(useMaterialActions);
    // @ts-ignore
    const componentInfo = await (0, util_1.getComponentsInfo)(appBuildDir, dependencies, fileCodeMap);
    const { widgets, dataBinds, componentSchema } = getComponentSchemaString(componentSchemaJson, false, componentInfo, processCssUnit);
    const pageClass = [rootPath, pageInstance.id].filter(Boolean).join('-');
    const pageStyleString = (0, weapps_core_1.toCssText)((0, cals_1.processCommonStyle2CSSProperties)(style, {
        defaultUnit: processCssUnit,
        rpxCalcRoot,
    }), (0, common_1.buildAsWebByBuildType)(buildTypeList) ? 'body' : 'page');
    fileCodeMap = generateFilesFromTemplate({
        templatesData: {
            [`pages/app.tpl.jsx|/pages/${pageInstance.id}/index.jsx`]: {
                pageUUID: rootPath ? `${rootPath}/${pageInstance.id}` : pageInstance.id,
                pageName: pageInstance.id,
                upperFirst: util_1.upperFirst,
                cleanVarName: util_1.cleanVarName,
                isSandbox,
                pageStyleText: pageStyleString,
                entryImports: entryImportStringArr.join(';\n'),
                componentImports: componentImportStringArr.join(';\n'),
                actionImports: actionImportStringArr.join(';\n'),
                useComponents: originComponentList,
                useMaterialActions,
                componentSchema,
                widgets,
                dataBinds,
                pageClass,
                // 复合组件预览需要
                isComposite: extraData.isComposite,
                compProps: extraData.compProps,
                title: data.navigationBarTitleText || data.title || '',
            },
            'pages/api.js': {},
        },
        baseTargetPath: dstDir,
        fileCodeMap,
    });
}
function getOriginComponentAndActionList(componentSchema, dependencies, originComponentList = [], originActionList = [], genericCompMap = {}) {
    const fieldSchema = new react_schema_renderer_1.Schema(componentSchema);
    if (fieldSchema.isObject()) {
        const { 'x-props': xProps } = fieldSchema;
        if (xProps) {
            const { listenerInstances, sourceKey } = xProps;
            // 属于抽象节点属性
            if (genericCompMap[sourceKey]) {
                const genericProps = genericCompMap[sourceKey];
                // @ts-ignore
                const genericComp = xProps.data[genericProps.propName];
                pullComponentToListByInstance(genericComp, originComponentList, dependencies);
            }
            pullComponentToListByInstance(sourceKey, originComponentList, dependencies);
            pullActionToListByInstances(listenerInstances, originActionList, dependencies);
        }
        const filedSchemaProperties = fieldSchema.properties || {};
        Object.keys(filedSchemaProperties).forEach((key) => {
            const schema = filedSchemaProperties[key];
            const schemaJson = schema;
            getOriginComponentAndActionList(schemaJson, dependencies, originComponentList, originActionList, genericCompMap);
        });
    }
    return {
        originComponentList,
        originActionList,
    };
}
exports.getOriginComponentAndActionList = getOriginComponentAndActionList;
function pullActionToListByInstances(listenerInstances, originActionList, fixedDependencies) {
    if (!listenerInstances || !listenerInstances.length) {
        return;
    }
    listenerInstances.map((pageListenerInstance) => {
        const { sourceKey, type } = pageListenerInstance;
        const { materialName, name, variableName } = (0, util_1.getMetaInfoBySourceKey)(sourceKey);
        const material = fixedDependencies.find((m) => m.name === materialName);
        const actionKey = `${materialName}_${name}`;
        const isExistAction = originActionList.find((item) => item.key === actionKey);
        if (!isExistAction) {
            originActionList.push({
                name,
                materialName,
                materialVersion: material === null || material === void 0 ? void 0 : material.version,
                key: actionKey,
                type,
                variableName,
                entries: material === null || material === void 0 ? void 0 : material.entries,
            });
        }
    });
}
exports.pullActionToListByInstances = pullActionToListByInstances;
function pullComponentToListByInstance(sourceKey, originComponentList, fixedDependencies) {
    var _a;
    const { materialName, name, variableName } = (0, util_1.getMetaInfoBySourceKey)(sourceKey);
    const componentKey = `${materialName}_${name}`;
    const isExistComponent = originComponentList.find((item) => item.key === componentKey);
    if (!isExistComponent) {
        const foundOne = fixedDependencies.find((m) => m.name === materialName);
        if (!foundOne)
            return;
        const { schemaVersion = '' } = foundOne;
        let isPlainProps = false;
        try {
            if (Number((_a = schemaVersion.split('.')) === null || _a === void 0 ? void 0 : _a[0]) >= 3) {
                isPlainProps = true;
            }
        }
        catch (e) { }
        originComponentList.push({
            sourceKey,
            name: name || '',
            materialName: materialName || '',
            materialVersion: foundOne.version,
            isComposite: foundOne.isComposite,
            key: componentKey,
            variableName: variableName || '',
            isPlainProps,
            entries: foundOne === null || foundOne === void 0 ? void 0 : foundOne.entries,
        });
    }
}
exports.pullComponentToListByInstance = pullComponentToListByInstance;
function isSlot(comp) {
    return comp.path && !comp['x-props'];
}
exports.isSlot = isSlot;
function getComponentSchemaString(componentSchema, isComposite = false, componentsInfoMap = {}, wrapperClass, processCssUnit, rpxCalcRoot = `var(${config_1.CALC_RPX_ROOT_VAL_NAME})`) {
    const componentInputProps = (0, util_1.getInputProps)(componentsInfoMap) || {};
    const copyJson = (0, util_1.simpleDeepClone)(componentSchema);
    const compWidgets = {};
    const compDataBinds = {};
    const processCommonStyle2CSSPropertiesOptions = {
        defaultUnit: processCssUnit,
        rpxCalcRoot,
    };
    const componentSchemaJson = (0, util_1.deepDealSchema)(copyJson, (schema) => {
        var _a, _b;
        const { 'x-props': xProps = {}, properties } = schema;
        const { dataBinds = [], commonStyle = {}, data = {}, classNameList = [], sourceKey, styleBind, classNameListBind, } = xProps;
        const componentInfo = componentsInfoMap[sourceKey];
        if (componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.selectableBlock) {
            schema.selectableBlock = componentInfo.selectableBlock;
        }
        if (componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.events) {
            schema.emitEvents = componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.events.map((item) => item.name);
        }
        else if (componentInfo === null || componentInfo === void 0 ? void 0 : componentInfo.emitEvents) {
            schema.emitEvents = componentInfo.emitEvents.map((item) => item.eventName);
        }
        // 生成 widgets/dataBinds
        if (!isSlot(schema) && schema.key) {
            const parentSchema = schema.parent;
            compWidgets[schema.key] = {
                ...data,
                style: (0, cals_1.processCommonStyle2CSSProperties)(commonStyle, processCommonStyle2CSSPropertiesOptions),
                classList: classNameList,
                widgetType: sourceKey,
                _parentId: isSlot(parentSchema) ? (_a = parentSchema.parent) === null || _a === void 0 ? void 0 : _a.key : parentSchema.key,
            };
            if (dataBinds.length > 0) {
                compDataBinds[schema.key] = generateDataBinds(dataBinds, isComposite);
            }
            if (styleBind) {
                if (!styleBind.bindDataPath) {
                    console.warn('无 bindDataPath', xProps);
                }
                else {
                    styleBind.propertyPath = 'style';
                    compDataBinds[schema.key] = {
                        ...(compDataBinds[schema.key] || {}),
                        ...generateDataBinds([styleBind], isComposite),
                    };
                }
            }
            if (classNameListBind) {
                classNameListBind.propertyPath = 'classList';
                compDataBinds[schema.key] = {
                    ...(compDataBinds[schema.key] || {}),
                    ...generateDataBinds([classNameListBind], isComposite),
                };
            }
        }
        // 抽离 widgets/dataBinds 后不再需要存储
        delete xProps.data;
        delete xProps.dataTypes;
        delete xProps.dataBinds;
        delete xProps.styleBind;
        delete xProps.classNameListBind;
        // 针对 JSON 体积做优化
        if (properties && (0, weapps_core_1.isEmptyObj)(properties)) {
            delete schema.properties;
        }
        delete schema.type;
        if (compWidgets[schema.key]) {
            const propsKeys = Object.keys(compWidgets[schema.key]);
            propsKeys.forEach((propsKey) => {
                const propsVaule = compWidgets[schema.key][propsKey];
                if (propsVaule && Array.isArray(propsVaule)) {
                    propsVaule.map((block) => {
                        var _a, _b;
                        const blockInstance = (block === null || block === void 0 ? void 0 : block.selectableBlock) && ((_b = (_a = block === null || block === void 0 ? void 0 : block.selectableBlock) === null || _a === void 0 ? void 0 : _a['x-props']) === null || _b === void 0 ? void 0 : _b.listenerInstances);
                        if (blockInstance) {
                            block.selectableBlock['x-props'].listenerInstances = generateListnerInstances(blockInstance, isComposite);
                        }
                        return block;
                    });
                }
            });
        }
        if (xProps) {
            // 如果是复合组件的根节点，则补充 wrapperClass
            if (isComposite) {
                if (!((_b = schema === null || schema === void 0 ? void 0 : schema.parent) === null || _b === void 0 ? void 0 : _b.parent)) {
                    if (!xProps.classNameList)
                        xProps.classNameList = [];
                    xProps.classNameList.push(wrapperClass);
                }
            }
            xProps.commonStyle = (0, cals_1.processCommonStyle2CSSProperties)(xProps.commonStyle, processCommonStyle2CSSPropertiesOptions);
            if ((0, weapps_core_1.isEmptyObj)(xProps.commonStyle)) {
                delete xProps.commonStyle;
            }
            if ((0, weapps_core_1.isEmptyObj)(xProps.style)) {
                delete xProps.style;
            }
            if (xProps.dataBinds && xProps.dataBinds.length === 0) {
                delete xProps.dataBinds;
            }
            if (xProps.listenerInstances && xProps.listenerInstances.length === 0) {
                delete xProps.listenerInstances;
            }
            if (xProps.data) {
                const xPropsData = xProps.data;
                if (xPropsData._waFor && xPropsData._waFor.length === 0) {
                    delete xPropsData._waFor;
                }
                if (xPropsData.title === '') {
                    delete xPropsData.title;
                }
                if ((0, weapps_core_1.isEmptyObj)(xPropsData)) {
                    delete xProps.data;
                }
            }
            if (xProps.listenerInstances) {
                xProps.listenerInstances = generateListnerInstances(xProps.listenerInstances, isComposite);
            }
            // 组件双向绑定
            const syncProps = componentInputProps[xProps.sourceKey];
            if (syncProps) {
                if (!xProps.listenerInstances)
                    xProps.listenerInstances = [];
                Object.keys(syncProps).forEach((key) => {
                    let syncPropArr = [];
                    const syncProp = syncProps[key];
                    // 统一转成数组处理
                    if (!Array.isArray(syncProp)) {
                        syncPropArr = [syncProp];
                    }
                    else {
                        syncPropArr = syncProp;
                    }
                    syncPropArr.forEach(({ changeEvent, valueFromEvent }) => {
                        // 双向绑定需要优先第一个执行，否则会出现输入框的拼音被打断的问题
                        xProps.listenerInstances.unshift({
                            trigger: changeEvent,
                            instanceFunction: `${config_1.REPLACE_SIGN}function({ event, forItems }) {
    const $for = forItems;
    const wid = ${isComposite ? 'this.widgets' : '$page.widgets'}.${schema.key};
    const widgetData = (forItems.forIndexes && forItems.forIndexes.length > 0) ? get(wid, forItems.forIndexes) : wid;
    widgetData.${key} = ${valueFromEvent};
  }.bind(this)${config_1.REPLACE_SIGN}`,
                        });
                    });
                });
            }
            if (xProps.dataBinds) {
                xProps.dataBinds = generateDataBinds(xProps.dataBinds, isComposite);
            }
            if (xProps.styleBind) {
                xProps.styleBind = generateDataBinds([xProps.styleBind], isComposite);
            }
            if (xProps.classNameListBind) {
                xProps.classNameListBind = generateDataBinds([xProps.classNameListBind], isComposite);
            }
        }
    });
    return {
        widgets: (0, util_1.JsonToStringWithVariableName)(compWidgets),
        dataBinds: (0, util_1.JsonToStringWithVariableName)(compDataBinds),
        componentSchema: (0, util_1.JsonToStringWithVariableName)(componentSchemaJson),
    };
}
exports.getComponentSchemaString = getComponentSchemaString;
// convert data binds to functions for performance & simplicity
function generateDataBinds(dataBinds, isComposite) {
    const dataBindFuncs = {};
    dataBinds.forEach((bind) => {
        if (!bind.bindDataPath) {
            return console.warn('无 bindDataPath', bind.propertyPath);
        }
        let funcCode = '()=>{}';
        if (bind.type === weapps_core_1.PropBindType.forItem) {
            funcCode = `(forItems) => forItems.${bind.bindDataPath}`;
        }
        else if (bind.type === weapps_core_1.PropBindType.scope) {
            funcCode = `() => ({__type: "scopedValue", getValue: ($scope)=>$scope.${bind.bindDataPath}})}`;
        }
        else if (bind.type === weapps_core_1.PropBindType.expression) {
            let imports = 'const $for = forItems;';
            let expression = bind.bindDataPath.replace(/;$/, '');
            try {
                imports += (0, util_1.generateExpressionAlias)(expression);
            }
            catch (e) {
                console.error('parse expression error', expression, e === null || e === void 0 ? void 0 : e.message);
            }
            if (isComposite) {
                expression = expression.replace(/\$comp/g, 'this.$WEAPPS_COMP');
            }
            const body = `${imports} return (\n${expression || 'undefined'}\n)`;
            if (isComposite) {
                funcCode = `(forItems) => {${body}}`;
            }
            else {
                funcCode = /\$scope\./.test(expression)
                    ? `(forItems, event) => ({__type: "scopedValue", getValue: ($scope) => {${body}}})`
                    : `(forItems, event) => {${body}}`;
            }
        }
        else if (bind.type === weapps_core_1.PropBindType.prop) {
            let { bindDataPath } = bind;
            const isNegated = bindDataPath.startsWith('!');
            if (isNegated)
                bindDataPath = bindDataPath.replace(/^!/, '');
            if (isComposite) {
                // 复合组件在预览时其实就是page，所以沿用page的变量即可
                funcCode = `() => ${isNegated ? '!' : ''}${'this.$WEAPPS_COMP'}.props.data.${bindDataPath}`;
            }
            else {
                // 复合组件在预览时其实就是page，所以沿用page的变量即可
                funcCode = `() => ${isNegated ? '!' : ''}$page.props.data.${bindDataPath}`;
            }
        }
        else {
            const PREFIX_MAP = {
                [weapps_core_1.PropBindType.state]: 'state',
                [weapps_core_1.PropBindType.computed]: 'computed',
                [weapps_core_1.PropBindType.dataVar]: 'dataVar',
                [weapps_core_1.PropBindType.stateData]: 'dataset.state',
                [weapps_core_1.PropBindType.paramData]: 'dataset.params',
            };
            switch (bind.type) {
                case weapps_core_1.PropBindType.state:
                case weapps_core_1.PropBindType.computed:
                case weapps_core_1.PropBindType.dataVar:
                case weapps_core_1.PropBindType.stateData:
                case weapps_core_1.PropBindType.paramData: {
                    const alias = PREFIX_MAP[bind.type];
                    if (bind.bindDataPath.startsWith('global.')) {
                        funcCode = bind.bindDataPath.replace(/^global\./, `app.${alias}.`);
                    }
                    else {
                        if (isComposite) {
                            funcCode = bind.bindDataPath
                                .replace(/^comp-\w+([.[])/, (_, $1) => `this.$WEAPPS_COMP.${alias}${$1}`)
                                .replace(/^\$\w+_\d+([.[])/, (_, $1) => `this.$WEAPPS_COMP.${alias}${$1}`);
                        }
                        else {
                            funcCode = bind.bindDataPath.replace(/^\w+([.[])/, (_, $1) => `$page.${alias}${$1}`);
                        }
                    }
                    funcCode = `() => ${funcCode}`;
                    break;
                }
            }
        }
        dataBindFuncs[bind.propertyPath] = `${config_1.REPLACE_SIGN}${funcCode}${config_1.REPLACE_SIGN}`;
    });
    return dataBindFuncs;
}
function generateDataBind(bind) {
    let expr = '';
    const { type, value } = bind;
    if (type === weapps_core_1.PropBindType.forItem) {
        expr = `forItems.${value}`;
    }
    else if (type === weapps_core_1.PropBindType.expression) {
        expr = value;
    }
    else if (type === weapps_core_1.PropBindType.prop) {
        const isNegated = value.startsWith('!');
        const originValue = value.replace(/^!/, '');
        expr = `${isNegated ? '!' : ''}$comp.props.data.${originValue}`;
    }
    else {
        const isGlobal = value.startsWith('global.');
        const isCopositive = value.startsWith('comp-') || value.startsWith('$comp_');
        const PREFIX_MAP = {
            [weapps_core_1.PropBindType.state]: 'state',
            [weapps_core_1.PropBindType.computed]: 'computed',
            [weapps_core_1.PropBindType.dataVar]: 'dataVar',
            [weapps_core_1.PropBindType.stateData]: 'dataset.state',
            [weapps_core_1.PropBindType.paramData]: 'dataset.params',
        };
        switch (type) {
            case weapps_core_1.PropBindType.state:
            case weapps_core_1.PropBindType.computed:
            case weapps_core_1.PropBindType.dataVar:
            case weapps_core_1.PropBindType.stateData:
            case weapps_core_1.PropBindType.paramData: {
                if (isCopositive) {
                    expr = value
                        .replace(/^comp-\w+/, `$comp.${PREFIX_MAP[type]}`)
                        .replace(/^\$comp_\w+/, `$comp.${PREFIX_MAP[type]}`);
                }
                else {
                    expr = value
                        .replace(/^\w+./, `${isGlobal ? 'app' : '$page'}.${PREFIX_MAP[type]}.`)
                        .replace(/^comp-\w+/, `$comp.${PREFIX_MAP[type]}`)
                        .replace(/^\$comp_\w+/, `$comp.${PREFIX_MAP[type]}`);
                }
            }
        }
    }
    return expr;
}
exports.generateDataBind = generateDataBind;
function generateListnerInstances(listenerInstances, isComposite = false) {
    return listenerInstances.map((listener) => {
        var _a, _b, _c, _d, _e;
        const generatedListener = {
            key: listener.key,
            trigger: listener.trigger,
            isCapturePhase: listener.isCapturePhase,
            noPropagation: listener.noPropagation,
        };
        if (listener.type === weapps_core_1.ActionType.Material) {
            const { sourceKey } = listener;
            const { variableName } = (0, util_1.getMetaInfoBySourceKey)(sourceKey);
            generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}${variableName}${config_1.REPLACE_SIGN}`;
        }
        else if (listener.type === weapps_core_1.ActionType.PropEvent) {
            if (isComposite) {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({ data, event, forItems }) {
          event.detail.data = data;
          this.props.emit('${((_a = listener.handler) === null || _a === void 0 ? void 0 : _a.name) || listener.jsCode}', event.detail);
        }.bind(this)${config_1.REPLACE_SIGN}`;
            }
            else {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function(...args) { $page.props.events.${((_b = listener.handler) === null || _b === void 0 ? void 0 : _b.name) || listener.jsCode}.apply(null, args) }${config_1.REPLACE_SIGN}`;
            }
        }
        else if (listener.type === weapps_core_1.ActionType.Inline) {
            // 内联 handler
            if (isComposite) {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({event, data, forItems}) {
          return ${listener.jsCode}
        }.bind(this)${config_1.REPLACE_SIGN}`;
            }
            else {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({event, data, forItems}) {
          return ${listener.jsCode}
        }.bind(pageCodeContext)${config_1.REPLACE_SIGN}`;
            }
        }
        else if (listener.type === weapps_core_1.ActionType.Platform) {
            let name = ((_c = listener.handler) === null || _c === void 0 ? void 0 : _c.name) || listener.jsCode;
            generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({data}) { return app.${name}(data) }${config_1.REPLACE_SIGN}`;
        }
        else if (listener.type === weapps_core_1.ActionType.DataSource) {
            generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}function({data}) { return app.cloud.callDataSource(data) }${config_1.REPLACE_SIGN}`;
        }
        else {
            // Lowcode action(handler)
            if (isComposite) {
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}this.handler.${((_d = listener.handler) === null || _d === void 0 ? void 0 : _d.name) || listener.jsCode}.bind(this)${config_1.REPLACE_SIGN}`;
            }
            else {
                let name = ((_e = listener.handler) === null || _e === void 0 ? void 0 : _e.name) || listener.jsCode;
                generatedListener.instanceFunction = `${config_1.REPLACE_SIGN}handler.${name}.bind(pageCodeContext)${config_1.REPLACE_SIGN}`;
            }
        }
        if (!(0, weapps_core_1.isEmptyObj)(listener.data)) {
            generatedListener.data = listener.data;
        }
        if (listener.dataBinds && listener.dataBinds.length > 0) {
            generatedListener.dataBinds = generateDataBinds(listener.dataBinds, isComposite);
        }
        return generatedListener;
    });
}
function getListenersString(listeners = [], isComposite = false) {
    return (0, util_1.JsonToStringWithVariableName)(generateListnerInstances(listeners, isComposite));
}
exports.getListenersString = getListenersString;
async function generateRouterFile(allAppDataList, appBuildDir, basename = '', buildTypeList = [], fileCodeMap, isSandbox, _indexPage = false) {
    const routerImports = [];
    const routerRenders = [];
    await Promise.all(allAppDataList.map(async (data) => {
        const { pageInstanceList, rootPath = '' } = data;
        const pageFilePath = rootPath ? `/src/packages/${rootPath}/` : '/src/';
        // 判断app环境才进行加载引入
        pageInstanceList.map((pageInstance) => {
            const pageId = [rootPath, pageInstance.id].filter((i) => !!i).join('_');
            const pagePath = rootPath ? `/${rootPath}/${pageInstance.id}` : `/${pageInstance.id}`;
            if (isSandbox) {
                routerImports.push(`import Page${pageId} from '${pageFilePath}pages/${pageInstance.id}/index';`);
            }
            else {
                routerImports.push(`import Page${pageId} from '${pageFilePath}pages/${pageInstance.id}/index';`);
                // routerImports.push(
                //   `const Page${pageId} = React.lazy(() => import('${pageFilePath}pages/${pageInstance.id}/index'));`
                // );
            }
            routerRenders.push({
                id: pageId,
                path: pagePath,
                isHome: !rootPath && !!pageInstance.isHome,
                root: rootPath || '',
                // Route 这里需要加 () => <Pageexample /> 才能触发热更新，具体原因还没查
                component: `${config_1.REPLACE_SIGN}() => <Page${pageId} />${config_1.REPLACE_SIGN}`,
            });
        });
    }));
    fileCodeMap = generateFilesFromTemplate({
        templatesData: {
            'router/index.tpl.jsx': {
                routerImports: routerImports.join('\n'),
                routerRenders: (0, util_1.JsonToStringWithVariableName)(routerRenders),
                basename,
                isHash: isSandbox,
                context: {
                    history: { _indexPage },
                    basic: (0, common_1.buildAsXPageByBuildType)(buildTypeList),
                    checkPageAuth: (0, common_1.buildAsXPageByBuildType)(buildTypeList) || (0, common_1.buildAsAdminPortalByBuildType)(buildTypeList)
                        ? 'function(){ return true }'
                        : 'undefined',
                },
            },
        },
        fileCodeMap,
        baseTargetPath: path_1.default.resolve(appBuildDir, 'src'),
    });
}
exports.generateRouterFile = generateRouterFile;
async function generateAppData(cals, dependencies, appBuildDir, deployOptions, fileCodeMap) {
    const usedComps = (0, cals_1.getUsedComps)([cals], dependencies);
    fileCodeMap = generateFilesFromTemplate({
        templatesData: {
            'router/data.tpl.js': {
                /**
                 * 不添加缩进以精简大小
                 */
                cals: JSON.stringify({
                    ...cals,
                    version: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.version) || '',
                    resources: undefined,
                    items: (cals.items || []).map((page) => {
                        return {
                            ...page,
                            resources: undefined,
                        };
                    }),
                    // 数据源配置单独生成
                    dataSources: [],
                    extra: {
                        ...(cals.extra || {}),
                        // 过滤设计态属性
                        pageTree: undefined,
                        compHiddenConfig: undefined,
                        componentPropsDatas: undefined,
                        processPages: undefined,
                        lastDependencies: undefined,
                        conditionalRendering: undefined,
                        dataForm: undefined,
                        compConfig: undefined,
                    },
                }),
                /**
                 * 不添加缩进以精简大小
                 */
                dependencies: JSON.stringify(dependencies
                    .filter((lib) => lib.name === config_1.OFFICIAL_LIB_KEY || !!usedComps[lib.name])
                    .map((lib) => {
                    // const usedCompNames = usedComps[lib.name];
                    return {
                        ...lib,
                        components: [],
                        actions: undefined,
                        srcZipUrl: undefined,
                        mpPkgUrl: undefined,
                        mainJsPath: undefined,
                        mainCssPath: undefined,
                        // components: lib.components.filter((comp) => usedCompNames?.has?.(comp.name)),
                    };
                })),
            },
        },
        fileCodeMap,
        baseTargetPath: path_1.default.resolve(appBuildDir, 'src'),
    });
}
exports.generateAppData = generateAppData;
async function writeLowCodeFiles(appData, appBuildDir, fileCodeMap, rootPath, processCssUnit, rpxCalcRoot) {
    var _a;
    const lowcodeRootDir = path_1.default.join(appBuildDir, 'lowcode');
    const codeModules = (0, cals_1.processRuntimeCodeResources)({ id: 'global' }, appData.codeModules, 'app');
    const themeCode = codeModules.find((mod) => mod.type === cals_1.ECodeType.THEME);
    await Promise.all(codeModules.map((m) => writeCode2file(m, 'global', undefined, processCssUnit, rpxCalcRoot)).concat());
    const styleModule = codeModules.find((mode) => mode.type === cals_1.ECodeType.STYLE);
    await writeCode2file({
        type: 'index',
        name: 'entry',
        path: 'global/index',
        code: `${['state', 'computed', 'lifecycle']
            .map((key) => `import ${key} from './${key}'`)
            .concat("import common from '../app/common'")
            .concat(appData.pageInstanceList.map((page) => `import * as $$_${page.id} from './${page.id}/index'`))
            .concat('')
            .join(';\n')}
export default {
  global: {
    state,
    computed,
    lifecycle,
    style: [{code:\`${styleModule
            ? ((_a = fileCodeMap[path_1.default.join(lowcodeRootDir, (0, weapps_core_1.getCodeModuleFilePath)('global', styleModule, {
                style: '.css',
            }))]) === null || _a === void 0 ? void 0 : _a.code) || ''
            : ''}\`}],
    common
  },
  ${appData.pageInstanceList.map((page) => `['${page.id}']: $$_${page.id}`).join(',\n')}
}
`,
    }, 'global', processCssUnit, rpxCalcRoot);
    await Promise.all(appData.pageInstanceList.map(async (page) => {
        var _a;
        const codeModules = (0, cals_1.processRuntimeCodeResources)({ id: page.id }, page.codeModules, 'page');
        await Promise.all(codeModules.map((m) => writeCode2file(m, page.id, page, processCssUnit, rpxCalcRoot)));
        const styleModule = codeModules.find((mode) => mode.type === cals_1.ECodeType.STYLE);
        await writeCode2file({
            type: 'index',
            name: 'entry',
            path: `${page.id}/index`,
            code: `export { default as state } from './state';
        export { default as  computed } from './computed';
        export { default as lifecycle } from './lifecycle';
        // export style from './style';
        export const style = [{code: \`${styleModule
                ? ((_a = fileCodeMap[path_1.default.join(lowcodeRootDir, (0, weapps_core_1.getCodeModuleFilePath)(page.id, styleModule, {
                    style: '.css',
                }))]) === null || _a === void 0 ? void 0 : _a.code) || ''
                : ''}\`}];
        export { $$_${page.id} as handler} from '../../app/handlers';
        `,
        }, 'global', processCssUnit, rpxCalcRoot);
    }));
    async function writeCode2file(mod, pageId, page, processCssUnit = 'rem', rpxCalcRoot) {
        const file = path_1.default.join(lowcodeRootDir, (0, weapps_core_1.getCodeModuleFilePath)(pageId, mod, {
            style: '.css',
        }));
        let code = mod.code || '';
        let weappsApiPrefix = '';
        if (mod.type !== cals_1.ECodeType.THEME && mod.type !== cals_1.ECodeType.STYLE && mod.type !== 'index') {
            const isIncludeAppDot = !!code
                // 先去掉模板代码
                .replace('通过 app.common.[name].xxx 访问这里定义的方法或值', '')
                // 再匹配
                .match(/[^'"\w]?app[^'"\w]/g);
            if (isIncludeAppDot) {
                // weappsApiPrefix = `import { app } from '${path
                //   .relative(path.dirname(file), `${appBuildDir}/app/global-api`)
                //   .replace(/\\/g, '/')}';\nconst $app = app;\n`;
                // weappsApiPrefix = `const app = window.app;\nconst $app = app;\n`;
                weappsApiPrefix = `${
                /* !isSandbox */ false
                    ? `import { app } from '${path_1.default
                        .relative(path_1.default.dirname(file), `${appBuildDir}/app/global-api`)
                        .replace(/\\/g, '/')}';`
                    : config_1.LOWCODE_INJECT_APP_TEMPLATE}\nconst $app = new Proxy({}, { get: function(obj, prop){ return app[prop] }});`;
            }
            if (page) {
                // weappsApiPrefix = `import { $page } from '${(path.dirname(file), appBuildDir).replace(
                //   /\\/g,
                //   '/',
                // )}/pages/${pageId}/api';`;
                // weappsApiPrefix = `cosnt $page = window.$page`;
            }
        }
        if (mod.type === 'style') {
            // 生成页面样式
            let pageStyleString = '';
            if (page) {
                const pageClass = [rootPath, page.id].filter(Boolean).join('-');
                pageStyleString = (0, weapps_core_1.toCssText)((0, cals_1.processCommonStyle2CSSProperties)(page.style, {
                    defaultUnit: processCssUnit,
                    rpxCalcRoot,
                }), `.weapps-page-${pageClass}`);
            }
            code = await (0, style_1.processLess)(((themeCode === null || themeCode === void 0 ? void 0 : themeCode.code) || '') + code, processCssUnit);
            // 页面类的样式通过注入css标签的方式挂载到body上
            // 此处不采用这种方式是为了防止 body.width 50% pagecontainer.width 50% 情况的出现
            // 期望是半屏，结果四分之一屏了
            // code = pageStyleString + os.EOL + code;
        }
        if (mod.type === 'theme') {
            return;
        }
        fileCodeMap[file] = {
            code: weappsApiPrefix + os_1.default.EOL + code,
        };
    }
}
exports.writeLowCodeFiles = writeLowCodeFiles;
async function generateCodeFromTpl(
/**
 * 应用ID
 */
appKey, 
/**
 * 应用数据
 */
appData, appBuildDir, rootPath, extraData, i18nConfig, fileCodeMap, isSandbox, subAppDataList, buildTypeList, deployOptions, 
/**
 * 静态域名
 */
domain, dependencies, processCssUnit, isPrivateMode) {
    var _a;
    const pageIds = [];
    const pageModules = {};
    appData.pageInstanceList.map((p) => {
        pageIds.push(p.id);
        pageModules[p.id] = p.codeModules;
    });
    const yyptConfig = await (0, util_1.getYyptConfigInfo)(extraData);
    const isXPage = (0, common_1.buildAsXPageByBuildType)(buildTypeList);
    const { staticDatasourceProfile = true } = (0, cals_1.parseVersion)('', dependencies);
    const genericCompMap = getGenericCompFromDep(dependencies);
    // originComponentList 包含了引入的组件与抽象节点绑定的组件
    const { originComponentList, originActionList } = appData.pageInstanceList.reduce((res, pageInstance) => {
        const { componentSchemaJson, listenerInstances: pageListenerInstances } = pageInstance;
        const { originComponentList = [], originActionList = [] } = getOriginComponentAndActionList(componentSchemaJson, dependencies, [], [], genericCompMap);
        pullActionToListByInstances(pageListenerInstances, originActionList, dependencies);
        return {
            originComponentList: res.originComponentList.concat(originComponentList),
            originActionList: res.originActionList.concat(originActionList),
        };
    }, {
        originComponentList: [],
        originActionList: [],
    });
    const useComponents = [];
    const usedCompMap = originComponentList.reduce((map, compItem) => {
        const sourceKey = `${compItem.materialName}:${compItem.name}`;
        if (!map[sourceKey]) {
            useComponents.push(compItem);
            map[sourceKey] = true;
        }
        return map;
    }, {});
    const originEntryMap = {};
    useComponents.forEach((component) => {
        if (!originEntryMap[component.materialName]) {
            const info = {
                materialName: component.materialName,
                version: component.materialVersion,
                entries: component.entries,
            };
            originEntryMap[component.materialName] = info;
        }
    });
    const useMaterialActions = [];
    originActionList.reduce((map, action) => {
        if (action.type === weapps_core_1.ActionType.Material) {
            const sourceKey = `${action.materialName}:${action.name}`;
            if (!map[sourceKey]) {
                useMaterialActions.push(action);
                map[sourceKey] = true;
            }
        }
        return map;
    }, {});
    const officialLib = dependencies.find((lib) => lib.name === config_1.OFFICIAL_LIB_KEY);
    useMaterialActions.forEach((action) => {
        if (!originEntryMap[action.materialName]) {
            const info = {
                materialName: action.materialName,
                version: action.materialVersion,
                entries: action.entries,
            };
            originEntryMap[action.materialName] = info;
        }
    });
    if (officialLib && !originEntryMap[config_1.OFFICIAL_LIB_KEY]) {
        originEntryMap[config_1.OFFICIAL_LIB_KEY] = {
            materialName: officialLib.name,
            version: officialLib.version,
            entries: officialLib.entries,
        };
    }
    const entryImportStringArr = isSandbox ? [] : getEntryImportStringArr(Object.values(originEntryMap));
    const componentImportStringArr = isSandbox
        ? useComponents.map((compItem) => {
            return `const ${(0, util_1.upperFirst)(compItem.variableName)} = window["@weapps-materials-main-${compItem.materialName}"].components["${compItem.name}"]`;
        })
        : getComponentImportStringArr(useComponents);
    const actionImportStringArr = isSandbox
        ? useMaterialActions.map((action) => {
            return `const ${action.variableName} = window["@weapps-materials-main-${action.materialName}"].actions["${action.name}"]`;
        })
        : getActionImportStringArr(useMaterialActions);
    unshiftAllOfficialActionToImportStringArr(officialLib, actionImportStringArr);
    // # all templates to be generated
    const templatesData = {
        /**
         * 使用 render 进行构建的话
         * 外部不再生成 app/global-api.js
         */
        // 'app/global-api.tpl.js': {
        //   appId: appKey,
        //   subPackageName: rootPath,
        //   domain,
        //   appConfig: JsonToStringWithVariableName({
        //     id: appKey,
        //     envId: appData.envId,
        //     clientId: appData.extra?.clientId,
        //     loginConfigVersion: appData.extra?.loginConfigVersion,
        //     staticResourceDomain: domain || '',
        //     envVersion: deployMode === DEPLOY_MODE.UPLOAD ? 'production' : 'preview',
        //     pages: appData.pageInstanceList
        //       .sort((item) => (item.isHome ? -1 : 1))
        //       .map((item) => ({
        //         id: item.id,
        //         type: (item as any).data?.isCustomLoginPage ? 'login' : '',
        //       })),
        //   }),
        // },
        'app/handlers.tpl.js': {
            pageModules,
        },
        'app/common.tpl.js': {
            mods: appData.codeModules
                .filter((m) => m.type === cals_1.ECodeType.NORMAL_MODULE && m.name !== cals_1.ECodeName.PLACEHOLDER)
                .map((m) => m.name),
        },
        'datasources/config.tpl.js': {
            appID: appKey,
            envID: appData.envId,
            isProd: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.mode) === types_1.DEPLOY_MODE.UPLOAD,
            clientID: (_a = appData.extra) === null || _a === void 0 ? void 0 : _a.clientId,
            isPrivateMode,
        },
        'datasources/datasource-profiles.tpl.js': {
            datasourceProfiles: JSON.stringify((0, dataSource_1.getDatasourceProfiles)(
            /**
             * 精简文件生成，新 cloud-sdk 不依赖 profile 了
             * 2022.08.24 设计态获取数据 outParams.propterties 运行态数据 outParams.items
             * 因此继续使用设计态数据
             * 2023.05.26 新版组件不再依赖此类数据，为了精简，判断不再注入
             */
            // [],
            staticDatasourceProfile ? appData.datasources || [] : []), null, 2),
        },
        'datasources/dataset-profiles.tpl.js': {
            datasetProfiles: (0, util_1.JsonToStringWithVariableName)((0, dataSource_1.getDatasetProfiles)(appData, [appData])),
        },
        'utils/adapter/dependencies.tpl.jsx': {
            upperFirst: util_1.upperFirst,
            OFFICIAL_LIB_KEY: config_1.OFFICIAL_LIB_KEY,
            entryImports: entryImportStringArr.join(';\n'),
            componentImports: componentImportStringArr.join(';\n'),
            actionImports: actionImportStringArr.join(';\n'),
            useComponents,
            useMaterialActions,
            controlMap: JSON.stringify(dependencies.reduce((map, lib) => {
                map[`${lib.name}@${lib.version}`] = {
                    schemaVersion: lib.schemaVersion,
                    components: lib.components
                        .filter((item) => {
                        return usedCompMap[`${lib.name}:${item.name}`];
                    })
                        .map((item) => {
                        const { name, dataForm, properties, emitEvents, events, meta, compConfig, inputProps, syncProps, isContainer, } = item;
                        const { name: metaName, title, platform, shortcut } = meta || {};
                        return {
                            name,
                            dataForm: Object.entries(dataForm || {}).reduce((map, [key, config]) => {
                                const { type, ['x-slot-scope']: xSlotScope, ['x-slot-directives']: xSlotDirectives, syncProp, inputProp, } = config;
                                const filteredDataForm = {
                                    type,
                                    'x-slot-scope': xSlotScope,
                                    'x-slot-directives': xSlotDirectives,
                                    syncProp,
                                    inputProp,
                                };
                                map[key] = filteredDataForm;
                                return map;
                            }, {}),
                            properties: properties
                                ? {
                                    ...properties,
                                    properties: Object.entries(properties.properties || {}).reduce((map, [key, config]) => {
                                        const { type, title } = config;
                                        map[key] = {
                                            type,
                                            title,
                                        };
                                        return map;
                                    }, {}),
                                }
                                : undefined,
                            emitEvents,
                            events,
                            meta: meta
                                ? {
                                    name: metaName,
                                    title,
                                    platform,
                                    shortcut,
                                }
                                : undefined,
                            compConfig,
                            inputProps,
                            syncProps,
                            isContainer,
                        };
                    }),
                };
                return map;
            }, {}), undefined, 2),
        },
    };
    if (!rootPath) {
        templatesData['index.jsx'] = {
            ...yyptConfig,
            i18nConfig,
            isSandbox,
            isBuildApp: buildTypeList.includes(common_1.BuildType.APP),
            subAppDataList,
            adminPortalKey: (deployOptions === null || deployOptions === void 0 ? void 0 : deployOptions.mode) === types_1.DEPLOY_MODE.UPLOAD ? appKey : `${appKey}-preview`,
            processCssUnit,
        };
    }
    fileCodeMap = generateFilesFromTemplate({
        templatesData,
        fileCodeMap,
        baseTargetPath: appBuildDir,
    });
}
exports.generateCodeFromTpl = generateCodeFromTpl;
function getComponentImportStringArr(components, componentImportStringArr = []) {
    components.map(async (component) => {
        const { name, materialName, materialVersion, variableName, entries } = component;
        // const fullName = `${materialName}_${name}`
        // 这里将头字母变成大写是为了能在jsx中以<XXX/>去引用组件
        let importString = '';
        if (entries === null || entries === void 0 ? void 0 : entries.components) {
            importString = `import { ${name} as ${(0, util_1.upperFirst)(variableName)} } from '${path_1.default.posix.join(`/src/libraries/${materialName}@${materialVersion}`, entries === null || entries === void 0 ? void 0 : entries.components)}'`;
        }
        else if (entries === null || entries === void 0 ? void 0 : entries.entry) {
            const componentsLibVariableName = (0, util_1.camelcase)(`${materialName}`);
            importString = `const { ${name}: ${(0, util_1.upperFirst)(variableName)} } = ${componentsLibVariableName}.components`;
        }
        else {
            importString = `import ${(0, util_1.upperFirst)(variableName)} from '/src/libraries/${`${materialName}@${materialVersion}`}/components/${name}'`;
        }
        if (!componentImportStringArr.includes(importString)) {
            componentImportStringArr.push(importString);
        }
    });
    return componentImportStringArr.filter((item) => !!item);
}
function getEntryImportStringArr(materialInfoList = [], entryImportStringArr = []) {
    materialInfoList.forEach(({ materialName, version, entries }) => {
        if (!((entries === null || entries === void 0 ? void 0 : entries.actions) && (entries === null || entries === void 0 ? void 0 : entries.components)) && (entries === null || entries === void 0 ? void 0 : entries.entry)) {
            const componentsLibVariableName = (0, util_1.camelcase)(`${materialName}`);
            const importComponentLibString = `import ${componentsLibVariableName} from '${path_1.default.posix.join(`/src/libraries/${materialName}@${version}`, entries === null || entries === void 0 ? void 0 : entries.entry)}'`;
            if (!entryImportStringArr.includes(importComponentLibString)) {
                entryImportStringArr.push(importComponentLibString);
            }
        }
    });
    return entryImportStringArr || [];
}
function getActionImportStringArr(originActionList, actionImportStringArr = []) {
    originActionList.map((action) => {
        if (action.type === weapps_core_1.ActionType.Material) {
            pushActionToImportStringArr(action, actionImportStringArr);
        }
    });
    return actionImportStringArr;
}
function pushActionToImportStringArr(listenerInstance, actionImportStringArr) {
    const { name, materialName, materialVersion, variableName, entries } = listenerInstance;
    const componentsLibVariableName = (0, util_1.camelcase)(`${materialName}`);
    const importString = (entries === null || entries === void 0 ? void 0 : entries.actions)
        ? `import { ${name} as ${variableName} } from '${path_1.default.posix.join(`/src/libraries/${materialName}@${materialVersion}`, entries === null || entries === void 0 ? void 0 : entries.actions)}'`
        : (entries === null || entries === void 0 ? void 0 : entries.entry)
            ? `const { ${name}: ${variableName} } = ${componentsLibVariableName}.actions`
            : `import ${variableName} from '/src/libraries/${materialName}@${materialVersion}/actions/${name}'`;
    if (actionImportStringArr.includes(importString)) {
        return;
    }
    actionImportStringArr.push(importString);
}
function unshiftAllOfficialActionToImportStringArr(officialLib, actionImportStringArr) {
    let importString = '';
    if (officialLib) {
        const { entries } = officialLib;
        const componentsLibVariableName = (0, util_1.camelcase)(`${officialLib.name}`);
        importString = (entries === null || entries === void 0 ? void 0 : entries.actions)
            ? `import * as $$OFFICIAL_ACTION from '${path_1.default.posix.join(`/src/libraries/${officialLib.name}@${officialLib.version}`, entries === null || entries === void 0 ? void 0 : entries.actions)}'`
            : (entries === null || entries === void 0 ? void 0 : entries.entry)
                ? `const $$OFFICIAL_ACTION  = ${componentsLibVariableName}.actions`
                : '';
        // TODO: 最早的散装函数需要全部手动引用
        // : `import ${variableName} from '/src/libraries/${materialName}@${materialVersion}/actions/${name}'`;
    }
    else {
        importString = `const $$OFFICIAL_ACTION = {}`;
    }
    if (actionImportStringArr.includes(importString)) {
        return;
    }
    actionImportStringArr.unshift(importString);
}
function generateFilesFromTemplate({ templatesData, fileCodeMap = {}, baseTargetPath = '/src', }) {
    // eslint-disable-next-line no-restricted-syntax
    for (let file in templatesData) {
        let [templatePath, targetPath] = file.split('|').filter((item) => item === null || item === void 0 ? void 0 : item.trim());
        if (!/^\/src\//.test(templatePath)) {
            templatePath = path_1.default.posix.join('/src', templatePath);
        }
        if (!targetPath) {
            targetPath = templatePath.replace(_1.TEMPLATE_REGEX, '$1');
        }
        if (/^\/src\//.test(targetPath)) {
            targetPath = path_1.default.relative('/src', targetPath);
        }
        if (!template_1.default[templatePath]) {
            throw new Error(`not found ${templatePath}`);
        }
        const tplStr = template_1.default[templatePath].code;
        const generatedCode = (0, lodash_1.template)(tplStr, { interpolate: /<%=([\s\S]+?)%>/g })(templatesData[file]);
        const outFile = path_1.default.join(baseTargetPath, targetPath);
        fileCodeMap[outFile] = {
            code: generatedCode,
        };
    }
    return fileCodeMap;
}
exports.generateFilesFromTemplate = generateFilesFromTemplate;
